﻿///////////////////////////////////////////////////////////////////////////////
//
// (C) 2015 ICE TEA GROUP LLC - ALL RIGHTS RESERVED
//
// 
//
// ALL INFORMATION CONTAINED HEREIN IS, AND REMAINS
// THE PROPERTY OF ICE TEA GROUP LLC AND ITS SUPPLIERS, IF ANY.
// THE INTELLECTUAL PROPERTY AND TECHNICAL CONCEPTS CONTAINED
// HEREIN ARE PROPRIETARY TO ICE TEA GROUP LLC AND ITS SUPPLIERS
// AND MAY BE COVERED BY U.S. AND FOREIGN PATENTS, PATENT IN PROCESS, AND
// ARE PROTECTED BY TRADE SECRET OR COPYRIGHT LAW.
//
// DISSEMINATION OF THIS INFORMATION OR REPRODUCTION OF THIS MATERIAL
// IS STRICTLY FORBIDDEN UNLESS PRIOR WRITTEN PERMISSION IS OBTAINED
// FROM ICE TEA GROUP LLC.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using Wisej.Base;
using Wisej.Core;

namespace Wisej.Ext.LocalStorage
{
	/// <summary>
	/// Provides access to the browser's local storage.
	/// </summary>
	[ToolboxItem(true)]
	public class LocalStorage : Wisej.Base.Component, IComponent
	{
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="T:Wisej.Ext.LocalStorage.LocalStorage" /> class.
		/// </summary>
		public LocalStorage()
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="T:Wisej.Ext.LocalStorage.LocalStorage" /> class together with the specified container.
		/// </summary>
		/// <param name="container">A <see cref="T:System.ComponentModel.IContainer" /> that represents the container for the timer. </param>
		public LocalStorage(IContainer container)
			: this()
		{
			if (container == null)
				throw new ArgumentNullException("container");

			container.Add(this);
		}

		#endregion

		#region Properties

		/// <summary>
		/// Indicates whether to use the session storage or the local storage.
		/// </summary>
		[DefaultValue(false)]
		[Description("Indicates whether to use the session storage or the local storage.")]
		public bool SessionStorage
		{
			get { return this._sessionStorage; }
			set
			{
				if (this._sessionStorage != value)
				{
					this._sessionStorage = value;
					Update();
				}
			}
		}
		private bool _sessionStorage;

		#endregion


		#region Methods

		/// <summary>
		/// Adds the specified key and value to the storage, or update that key's value if it already exists.
		/// </summary>
		/// <param name="key">A string containing the name of the key you want to create/update.</param>
		/// <param name="value">A string containing the value you want to give the key you are creating/updating.</param>
		public void SetItem(string key, string value)
		{
			if (key == null)
				throw new ArgumentNullException("key");

			if (value == null)
				throw new ArgumentNullException("value");

			Call("setItem", key, value);
		}

		/// <summary>
		/// Returns the value associated with the specified key.
		/// </summary>
		/// <param name="key">A string containing the name of the key you want to retrieve the value of.</param>
		/// <param name="callback">A callback function that will receive the value associated with the specified key.</param>
		public void GetItem(string key, Action<string> callback)
		{
			if (key == null)
				throw new ArgumentNullException("key");

			if (callback == null)
				throw new ArgumentNullException("callback");

			// save the callback in the dictionary and issue a getItem call
			// using the hash of the callback object to identify the async response.
			if (this._callbacks == null)
				this._callbacks = new Dictionary<int, Action<string>>();

			int id = callback.GetHashCode();
			this._callbacks[id] = callback;

			Call("getItem", key, id);
		}
		private Dictionary<int, Action<string>> _callbacks = null;

		/// <summary>
		/// Removes the specified key from storage.
		/// </summary>
		/// <param name="key">A string containing the name of the key you want to remove.</param>
		public void RemoveItem(string key)
		{
			if (key == null)
				throw new ArgumentNullException("key");

			Call("removeItem", key);
		}

		/// <summary>
		/// Clears all stored keys.
		/// </summary>
		public void Clear()
		{
			Call("clear");
		}

		#endregion

		#region IComponent

		/// <summary>
		/// Returns or sets the <see cref="T:System.ComponentModel.ISite" /> associated with 
		/// the <see cref="T:System.ComponentModel.IComponent" />.
		/// </summary>
		/// <returns>The <see cref="T:System.ComponentModel.ISite" /> object associated with the component; or null, if the component does not have a site.</returns>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public virtual ISite Site
		{
			get { return this._site; }
			set
			{
				this._site = value;
				((IWisejComponent)this).DesignMode = value == null ? false : value.DesignMode;
			}
		}
		private ISite _site;

		/// <summary>
		/// Returns a value that indicates whether the <see cref="T:System.ComponentModel.IComponent" /> is currently in design mode.
		/// </summary>
		/// <returns>true if the <see cref="T:System.ComponentModel.IComponent" /> is in design mode; otherwise, false.</returns>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		protected bool DesignMode
		{
			get { return this._site != null && this._site.DesignMode; }
		}

		#endregion

		#region Wisej Implementation

		// Process the getCurrentPosition response from the client.
		private void ProcessCallbackWebEvent(WisejEventArgs e)
		{
			dynamic data = e.Parameters.Data;

			// find the corresponding request.
			if (this._callbacks != null)
			{
				int id = data.id ?? 0;
				Action<string> callback = null;
				if (this._callbacks.TryGetValue(id, out callback))
				{
					this._callbacks.Remove(id);
					callback(data.value as string);
				}
			}
		}

		/// <summary>
		/// Processes the event from the client.
		/// </summary>
		/// <param name="e">Event arguments.</param>
		protected override void OnWebEvent(WisejEventArgs e)
		{
			switch (e.Type)
			{
				case "callback":
					ProcessCallbackWebEvent(e);
					break;

				default:
					base.OnWebEvent(e);
					break;
			}
		}

		/// <summary>
		/// Renders the client component.
		/// </summary>
		/// <param name="config">Dynamic configuration object.</param>
		protected override void OnWebRender(dynamic config)
		{
			base.OnWebRender((object)config);

			config.className = "wisej.ext.LocalStorage";
			config.sessionStorage = this.SessionStorage;

			WiredEvents events = new WiredEvents();
			events.Add("callback(Data)");
			config.wiredEvents = events;
		}

		#endregion

	}
}